@@ -14,7 +14,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclCmdIL.c,v 1.15 1999/08/10 17:35:18 redman Exp $
+ * RCS: @(#) $Id: tclCmdIL.c,v 1.24.2.3 2001/10/08 15:50:24 dkf Exp $
  */
 
 #include "tclInt.h"
@@ -30,6 +30,7 @@
 
 typedef struct SortElement {
     Tcl_Obj *objPtr;			/* Object being sorted. */
+    int count;				/* number of same elements in list */
     struct SortElement *nextPtr;        /* Next element in the list, or
 					 * NULL for end of list. */
 } SortElement;
@@ -71,6 +72,13 @@ typedef struct SortInfo {
 #define SORTMODE_COMMAND    3
 #define SORTMODE_DICTIONARY 4
 
+/*
+ * Magic values for the index field of the SortInfo structure.
+ * Note that the index "end-1" will be translated to SORTIDX_END-1, etc.
+ */
+#define SORTIDX_NONE	-1		/* Not indexed; use whole value. */
+#define SORTIDX_END	-2		/* Indexed from end. */
+
 /*
  * Forward declarations for procedures defined in this file:
  */
@@ -465,7 +473,7 @@ Tcl_InfoObjCmd(clientData, interp, objc, objv)
  *          info args procName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -527,7 +535,7 @@ InfoArgsCmd(dummy, interp, objc, objv)
  *          info body procName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -561,24 +569,17 @@ InfoBodyCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
 
-    /*
-     * we need to check if the body from this procedure had been generated
-     * from a precompiled body. If that is the case, then the bodyPtr's
-     * string representation is bogus, since sources are not available.
-     * In order to make sure that later manipulations of the object do not
-     * invalidate the internal representation, we make a copy of the string
-     * representation and return that one, instead.
+    /* 
+     * Here we used to return procPtr->bodyPtr, except when the body was
+     * bytecompiled - in that case, the return was a copy of the body's
+     * string rep. In order to better isolate the implementation details
+     * of the compiler/engine subsystem, we now always return a copy of 
+     * the string rep. It is important to return a copy so that later 
+     * manipulations of the object do not invalidate the internal rep.
      */
 
     bodyPtr = procPtr->bodyPtr;
-    resultPtr = bodyPtr;
-    if (bodyPtr->typePtr == &tclByteCodeType) {
-        ByteCode *codePtr = (ByteCode *) bodyPtr->internalRep.otherValuePtr;
-
-        if (codePtr->flags & TCL_BYTECODE_PRECOMPILED) {
-            resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
-        }
-    }
+    resultPtr = Tcl_NewStringObj(bodyPtr->bytes, bodyPtr->length);
     
     Tcl_SetObjResult(interp, resultPtr);
     return TCL_OK;
@@ -596,7 +597,7 @@ InfoBodyCmd(dummy, interp, objc, objv)
  *          info cmdcount
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -638,7 +639,7 @@ InfoCmdCountCmd(dummy, interp, objc, objv)
  *          info commands ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -764,7 +765,7 @@ InfoCommandsCmd(dummy, interp, objc, objv)
  *          info complete command
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -806,7 +807,7 @@ InfoCompleteCmd(dummy, interp, objc, objv)
  *          info default procName arg varName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -890,7 +891,7 @@ InfoDefaultCmd(dummy, interp, objc, objv)
  *          info exists varName
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -936,7 +937,7 @@ InfoExistsCmd(dummy, interp, objc, objv)
  *          info globals ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1002,7 +1003,7 @@ InfoGlobalsCmd(dummy, interp, objc, objv)
  *          info hostname
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1046,7 +1047,7 @@ InfoHostnameCmd(dummy, interp, objc, objv)
  *          info level ?number?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1120,7 +1121,7 @@ InfoLevelCmd(dummy, interp, objc, objv)
  *          info library
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1165,7 +1166,7 @@ InfoLibraryCmd(dummy, interp, objc, objv)
  *          info loaded ?interp?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1210,7 +1211,7 @@ InfoLoadedCmd(dummy, interp, objc, objv)
  *          info locals ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1339,7 +1340,7 @@ AppendLocals(interp, listPtr, pattern, includeLinks)
  *          info nameofexecutable
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1382,7 +1383,7 @@ InfoNameOfExecutableCmd(dummy, interp, objc, objv)
  *          info patchlevel
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1419,14 +1420,17 @@ InfoPatchLevelCmd(dummy, interp, objc, objv)
  *
  * InfoProcsCmd --
  *
- *      Called to implement the "info procs" command that returns the
- *      procedures in the current namespace that match an optional pattern.
- *      Handles the following syntax:
+ *	Called to implement the "info procs" command that returns the
+ *	list of procedures in the interpreter that match an optional pattern.
+ *	The pattern, if any, consists of an optional sequence of namespace
+ *	names separated by "::" qualifiers, which is followed by a
+ *	glob-style pattern that restricts which commands are returned.
+ *	Handles the following syntax:
  *
  *          info procs ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1442,50 +1446,126 @@ InfoProcsCmd(dummy, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    char *cmdName, *pattern;
-    Namespace *currNsPtr = (Namespace *) Tcl_GetCurrentNamespace(interp);
+    char *cmdName, *pattern, *simplePattern;
+    Namespace *nsPtr;
+#ifdef INFO_PROCS_SEARCH_GLOBAL_NS
+    Namespace *globalNsPtr = (Namespace *) Tcl_GetGlobalNamespace(interp);
+#endif
+    Namespace *currNsPtr   = (Namespace *) Tcl_GetCurrentNamespace(interp);
+    Tcl_Obj *listPtr, *elemObjPtr;
+    int specificNsInPattern = 0;  /* Init. to avoid compiler warning. */
     register Tcl_HashEntry *entryPtr;
     Tcl_HashSearch search;
     Command *cmdPtr, *realCmdPtr;
-    Tcl_Obj *listPtr;
+
+    /*
+     * Get the pattern and find the "effective namespace" in which to
+     * list procs.
+     */
 
     if (objc == 2) {
-        pattern = NULL;
+	simplePattern = NULL;
+	nsPtr = currNsPtr;
+	specificNsInPattern = 0;
     } else if (objc == 3) {
-        pattern = Tcl_GetString(objv[2]);
+	/*
+	 * From the pattern, get the effective namespace and the simple
+	 * pattern (no namespace qualifiers or ::'s) at the end. If an
+	 * error was found while parsing the pattern, return it. Otherwise,
+	 * if the namespace wasn't found, just leave nsPtr NULL: we will
+	 * return an empty list since no commands there can be found.
+	 */
+
+	Namespace *dummy1NsPtr, *dummy2NsPtr;
+
+	pattern = Tcl_GetString(objv[2]);
+	TclGetNamespaceForQualName(interp, pattern, (Namespace *) NULL,
+		/*flags*/ 0, &nsPtr, &dummy1NsPtr, &dummy2NsPtr,
+		&simplePattern);
+
+	if (nsPtr != NULL) {	/* we successfully found the pattern's ns */
+	    specificNsInPattern = (strcmp(simplePattern, pattern) != 0);
+	}
     } else {
         Tcl_WrongNumArgs(interp, 2, objv, "?pattern?");
         return TCL_ERROR;
     }
 
     /*
-     * Scan through the current namespace's command table and return a list
-     * of all procs that match the pattern.
+     * Scan through the effective namespace's command table and create a
+     * list with all procs that match the pattern. If a specific
+     * namespace was requested in the pattern, qualify the command names
+     * with the namespace name.
      */
-    
+
     listPtr = Tcl_NewListObj(0, (Tcl_Obj **) NULL);
-    for (entryPtr = Tcl_FirstHashEntry(&currNsPtr->cmdTable, &search);
-            entryPtr != NULL;
-            entryPtr = Tcl_NextHashEntry(&search)) {
-        cmdName = Tcl_GetHashKey(&currNsPtr->cmdTable, entryPtr);
-        cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
+    if (nsPtr != NULL) {
+	entryPtr = Tcl_FirstHashEntry(&nsPtr->cmdTable, &search);
+	while (entryPtr != NULL) {
+	    cmdName = Tcl_GetHashKey(&nsPtr->cmdTable, entryPtr);
+	    if ((simplePattern == NULL)
+	            || Tcl_StringMatch(cmdName, simplePattern)) {
+		cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
+
+		realCmdPtr = (Command *)
+		    TclGetOriginalCommand((Tcl_Command) cmdPtr);
+
+		if (TclIsProc(cmdPtr)
+		        || ((realCmdPtr != NULL) && TclIsProc(realCmdPtr))) {
+		    if (specificNsInPattern) {
+			elemObjPtr = Tcl_NewObj();
+			Tcl_GetCommandFullName(interp, (Tcl_Command) cmdPtr,
+				elemObjPtr);
+		    } else {
+			elemObjPtr = Tcl_NewStringObj(cmdName, -1);
+		    }
+
+		    Tcl_ListObjAppendElement(interp, listPtr, elemObjPtr);
+		}
+	    }
+	    entryPtr = Tcl_NextHashEntry(&search);
+	}
 
 	/*
-	 * If the command isn't itself a proc, it still might be an
-	 * imported command that points to a "real" proc in a different
-	 * namespace.
+	 * If the effective namespace isn't the global :: namespace, and a
+	 * specific namespace wasn't requested in the pattern, then add in
+	 * all global :: procs that match the simple pattern. Of course,
+	 * we add in only those procs that aren't hidden by a proc in
+	 * the effective namespace.
 	 */
 
-	realCmdPtr = (Command *) TclGetOriginalCommand(
-	        (Tcl_Command) cmdPtr);
-        if (TclIsProc(cmdPtr)
-	        || ((realCmdPtr != NULL) && TclIsProc(realCmdPtr))) {
-            if ((pattern == NULL) || Tcl_StringMatch(cmdName, pattern)) {
-                Tcl_ListObjAppendElement(interp, listPtr,
-		        Tcl_NewStringObj(cmdName, -1));
-            }
-        }
+#ifdef INFO_PROCS_SEARCH_GLOBAL_NS
+	/*
+	 * If "info procs" worked like "info commands", returning the
+	 * commands also seen in the global namespace, then you would
+	 * include this code.  As this could break backwards compatibilty
+	 * with 8.0-8.2, we decided not to "fix" it in 8.3, leaving the
+	 * behavior slightly different.
+	 */
+	if ((nsPtr != globalNsPtr) && !specificNsInPattern) {
+	    entryPtr = Tcl_FirstHashEntry(&globalNsPtr->cmdTable, &search);
+	    while (entryPtr != NULL) {
+		cmdName = Tcl_GetHashKey(&globalNsPtr->cmdTable, entryPtr);
+		if ((simplePattern == NULL)
+	                || Tcl_StringMatch(cmdName, simplePattern)) {
+		    if (Tcl_FindHashEntry(&nsPtr->cmdTable, cmdName) == NULL) {
+			cmdPtr = (Command *) Tcl_GetHashValue(entryPtr);
+			realCmdPtr = (Command *) TclGetOriginalCommand(
+			        (Tcl_Command) cmdPtr);
+
+			if (TclIsProc(cmdPtr) || ((realCmdPtr != NULL)
+				&& TclIsProc(realCmdPtr))) {
+			    Tcl_ListObjAppendElement(interp, listPtr,
+			            Tcl_NewStringObj(cmdName, -1));
+			}
+		    }
+		}
+		entryPtr = Tcl_NextHashEntry(&search);
+	    }
+	}
+#endif
     }
+
     Tcl_SetObjResult(interp, listPtr);
     return TCL_OK;
 }
@@ -1502,7 +1582,7 @@ InfoProcsCmd(dummy, interp, objc, objv)
  *          info script
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1542,7 +1622,7 @@ InfoScriptCmd(dummy, interp, objc, objv)
  *          info sharedlibextension
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1580,7 +1660,7 @@ InfoSharedlibCmd(dummy, interp, objc, objv)
  *          info tclversion
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1627,7 +1707,7 @@ InfoTclVersionCmd(dummy, interp, objc, objv)
  *          info vars ?pattern?
  *
  * Results:
- *      Returns TCL_OK is successful and TCL_ERROR is there is an error.
+ *      Returns TCL_OK if successful and TCL_ERROR if there is an error.
  *
  * Side effects:
  *      Returns a result in the interpreter's result object. If there is
@@ -1757,7 +1837,7 @@ InfoVarsCmd(dummy, interp, objc, objv)
 		entryPtr = Tcl_NextHashEntry(&search);
 	    }
 	}
-    } else {
+    } else if (((Interp *)interp)->varFramePtr->procPtr != NULL) {
 	AppendLocals(interp, listPtr, simplePattern, 1);
     }
     
@@ -1940,8 +2020,7 @@ Tcl_LinsertObjCmd(dummy, interp, objc, objv)
     register int objc;		/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
-    Tcl_Obj *listPtr, *resultPtr;
-    Tcl_ObjType *typePtr;
+    Tcl_Obj *listPtr;
     int index, isDuplicate, len, result;
    
     if (objc < 4) {
@@ -1954,69 +2033,58 @@ Tcl_LinsertObjCmd(dummy, interp, objc, objv)
      * will invalidate the list's internal representation.
      */
 
-    result = TclGetIntForIndex(interp, objv[2], /*endValue*/ INT_MAX,
-	    &index);
+    result = Tcl_ListObjLength(interp, objv[1], &len);
     if (result != TCL_OK) {
 	return result;
     }
 
     /*
-     * If the list object is unshared we can modify it directly. Otherwise
-     * we create a copy to modify: this is "copy on write". We create the
-     * duplicate directly in the interpreter's object result.
+     * Get the index.  "end" is interpreted to be the index after the last
+     * element, such that using it will cause any inserted elements to be
+     * appended to the list.
      */
-    
+
+    result = TclGetIntForIndex(interp, objv[2], /*end*/ len, &index);
+    if (result != TCL_OK) {
+	return result;
+    }
+    if (index > len) {
+	index = len;
+    }
+
+    /*
+     * If the list object is unshared we can modify it directly. Otherwise
+     * we create a copy to modify: this is "copy on write".
+     */
+
     listPtr = objv[1];
     isDuplicate = 0;
     if (Tcl_IsShared(listPtr)) {
-	/*
-	 * The following code must reflect the logic in Tcl_DuplicateObj()
-	 * except that it must duplicate the list object directly into the
-	 * interpreter's result.
-	 */
-	
-	Tcl_ResetResult(interp);
-	resultPtr = Tcl_GetObjResult(interp);
-	typePtr = listPtr->typePtr;
-	if (listPtr->bytes == NULL) {
-	    resultPtr->bytes = NULL;
-	} else if (listPtr->bytes != tclEmptyStringRep) {
-	    len = listPtr->length;
-	    TclInitStringRep(resultPtr, listPtr->bytes, len);
-	}
-	if (typePtr != NULL) {
-	    if (typePtr->dupIntRepProc == NULL) {
-		resultPtr->internalRep = listPtr->internalRep;
-		resultPtr->typePtr = typePtr;
-	    } else {
-		(*typePtr->dupIntRepProc)(listPtr, resultPtr);
-	    }
-	}
-	listPtr = resultPtr;
+	listPtr = Tcl_DuplicateObj(listPtr);
 	isDuplicate = 1;
     }
-    
-    if ((objc == 4) && (index == INT_MAX)) {
+
+    if ((objc == 4) && (index == len)) {
 	/*
 	 * Special case: insert one element at the end of the list.
 	 */
-
 	result = Tcl_ListObjAppendElement(interp, listPtr, objv[3]);
     } else if (objc > 3) {
 	result = Tcl_ListObjReplace(interp, listPtr, index, 0,
 				    (objc-3), &(objv[3]));
     }
     if (result != TCL_OK) {
+	if (isDuplicate) {
+	    Tcl_DecrRefCount(listPtr); /* free unneeded obj */
+	}
 	return result;
     }
-    
+
     /*
      * Set the interpreter's object result.
      */
 
-    if (!isDuplicate) {
-	Tcl_SetObjResult(interp, listPtr);
-    }
+    Tcl_SetObjResult(interp, listPtr);
     return TCL_OK;
 }
 
@@ -2223,9 +2291,7 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
     Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     register Tcl_Obj *listPtr;
-    int createdNewObj, first, last, listLen, numToDelete;
-    int firstArgLen, result;
-    char *firstArg;
+    int isDuplicate, first, last, listLen, numToDelete, result;
 
     if (objc < 4) {
 	Tcl_WrongNumArgs(interp, 1, objv,
@@ -2233,53 +2299,43 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
 	return TCL_ERROR;
     }
 
-    /*
-     * If the list object is unshared we can modify it directly, otherwise
-     * we create a copy to modify: this is "copy on write".
-     */
-    
-    listPtr = objv[1];
-    createdNewObj = 0;
-    if (Tcl_IsShared(listPtr)) {
-	listPtr = Tcl_DuplicateObj(listPtr);
-	createdNewObj = 1;
-    }
-    result = Tcl_ListObjLength(interp, listPtr, &listLen);
+    result = Tcl_ListObjLength(interp, objv[1], &listLen);
     if (result != TCL_OK) {
-        errorReturn:
-	if (createdNewObj) {
-	    Tcl_DecrRefCount(listPtr); /* free unneeded obj */
-	}
 	return result;
     }
 
     /*
-     * Get the first and last indexes.
+     * Get the first and last indexes.  "end" is interpreted to be the index
+     * for the last element, such that using it will cause that element to
+     * be included for deletion.
      */
 
-    result = TclGetIntForIndex(interp, objv[2], /*endValue*/ (listLen - 1),
-	    &first);
+    result = TclGetIntForIndex(interp, objv[2], /*end*/ (listLen - 1), &first);
     if (result != TCL_OK) {
-	goto errorReturn;
+	return result;
     }
-    firstArg = Tcl_GetStringFromObj(objv[2], &firstArgLen);
 
-    result = TclGetIntForIndex(interp, objv[3], /*endValue*/ (listLen - 1),
-	    &last);
+    result = TclGetIntForIndex(interp, objv[3], /*end*/ (listLen - 1), &last);
     if (result != TCL_OK) {
-	goto errorReturn;
+	return result;
     }
 
     if (first < 0)  {
     	first = 0;
     }
-    if ((first >= listLen) && (listLen > 0)
-	    && (strncmp(firstArg, "end", (unsigned) firstArgLen) != 0)) {
+
+    /*
+     * Complain if the user asked for a start element that is greater than the
+     * list length.  This won't ever trigger for the "end*" case as that will
+     * be properly constrained by TclGetIntForIndex because we use listLen-1
+     * (to allow for replacing the last elem).
+     */
+
+    if ((first >= listLen) && (listLen > 0)) {
 	Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
 		"list doesn't contain element ",
 		Tcl_GetString(objv[2]), (int *) NULL);
-	result = TCL_ERROR;
-	goto errorReturn;
+	return TCL_ERROR;
     }
     if (last >= listLen) {
     	last = (listLen - 1);
@@ -2290,6 +2346,17 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
 	numToDelete = 0;
     }
 
+    /*
+     * If the list object is unshared we can modify it directly, otherwise
+     * we create a copy to modify: this is "copy on write".
+     */
+
+    listPtr = objv[1];
+    isDuplicate = 0;
+    if (Tcl_IsShared(listPtr)) {
+	listPtr = Tcl_DuplicateObj(listPtr);
+	isDuplicate = 1;
+    }
     if (objc > 4) {
 	result = Tcl_ListObjReplace(interp, listPtr, first, numToDelete,
 	        (objc-4), &(objv[4]));
@@ -2298,7 +2365,10 @@ Tcl_LreplaceObjCmd(dummy, interp, objc, objv)
 		0, NULL);
     }
     if (result != TCL_OK) {
-	goto errorReturn;
+	if (isDuplicate) {
+	    Tcl_DecrRefCount(listPtr); /* free unneeded obj */
+	}
+	return result;
     }
 
     /*
@@ -2424,7 +2494,7 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
     int objc;			/* Number of arguments. */
     Tcl_Obj *CONST objv[];	/* Argument values. */
 {
-    int i, index;
+    int i, index, unique;
     Tcl_Obj *resultPtr;
     int length;
     Tcl_Obj *cmdPtr, **listObjPtrs;
@@ -2433,9 +2503,10 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
     SortInfo sortInfo;                  /* Information about this sort that
                                          * needs to be passed to the 
                                          * comparison function */
-    static char *switches[] =
-	    {"-ascii", "-command", "-decreasing", "-dictionary",
-	    "-increasing", "-index", "-integer", "-real", (char *) NULL};
+    static char *switches[] = {
+	"-ascii", "-command", "-decreasing", "-dictionary", "-increasing",
+	"-index", "-integer", "-real", "-unique", (char *) NULL
+    };
 
     resultPtr = Tcl_GetObjResult(interp);
     if (objc < 2) {
@@ -2449,10 +2520,11 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 
     sortInfo.isIncreasing = 1;
     sortInfo.sortMode = SORTMODE_ASCII;
-    sortInfo.index = -1;
+    sortInfo.index = SORTIDX_NONE;
     sortInfo.interp = interp;
     sortInfo.resultCode = TCL_OK;
     cmdPtr = NULL;
+    unique = 0;
     for (i = 1; i < objc-1; i++) {
 	if (Tcl_GetIndexFromObj(interp, objv[i], switches, "option", 0, &index)
 		!= TCL_OK) {
@@ -2489,11 +2561,10 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 			    -1);
 		    return TCL_ERROR;
 		}
-		if (TclGetIntForIndex(interp, objv[i+1], -2, &sortInfo.index)
-			!= TCL_OK) {
+		if (TclGetIntForIndex(interp, objv[i+1], SORTIDX_END,
+			&sortInfo.index) != TCL_OK) {
 		    return TCL_ERROR;
 		}
-		cmdPtr = objv[i+1];
 		i++;
 		break;
 	    case 6:			/* -integer */
@@ -2502,6 +2573,9 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 	    case 7:			/* -real */
 		sortInfo.sortMode = SORTMODE_REAL;
 		break;
+	    case 8:			/* -unique */
+		unique = 1;
+		break;
 	}
     }
     if (sortInfo.sortMode == SORTMODE_COMMAND) {
@@ -2512,27 +2586,29 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 	 */
 
         Tcl_Obj *newCommandPtr = Tcl_DuplicateObj(cmdPtr);
+	Tcl_Obj *newObjPtr = Tcl_NewObj();
 
-	if (Tcl_ListObjAppendElement(interp, newCommandPtr, Tcl_NewObj()) 
-	        != TCL_OK) {
+	Tcl_IncrRefCount(newCommandPtr);
+	if (Tcl_ListObjAppendElement(interp, newCommandPtr, newObjPtr)
+		!= TCL_OK) {
+	    Tcl_DecrRefCount(newCommandPtr);
+	    Tcl_IncrRefCount(newObjPtr);
+	    Tcl_DecrRefCount(newObjPtr);
 	    return TCL_ERROR;
 	}
 	Tcl_ListObjAppendElement(interp, newCommandPtr, Tcl_NewObj());
 	sortInfo.compareCmdPtr = newCommandPtr;
-	Tcl_IncrRefCount(newCommandPtr);
     }
 
     sortInfo.resultCode = Tcl_ListObjGetElements(interp, objv[objc-1],
 	    &length, &listObjPtrs);
-    if (sortInfo.resultCode != TCL_OK) {
+    if (sortInfo.resultCode != TCL_OK || length <= 0) {
 	goto done;
     }
-    if (length <= 0) {
-        return TCL_OK;
-    }
     elementArray = (SortElement *) ckalloc(length * sizeof(SortElement));
     for (i=0; i < length; i++){
 	elementArray[i].objPtr = listObjPtrs[i];
+	elementArray[i].count = 0;
 	elementArray[i].nextPtr = &elementArray[i+1];
     }
     elementArray[length-1].nextPtr = NULL;
@@ -2545,8 +2621,18 @@ Tcl_LsortObjCmd(clientData, interp, objc, objv)
 
 	Tcl_ResetResult(interp);
 	resultPtr = Tcl_GetObjResult(interp);
-	for (; elementPtr != NULL; elementPtr = elementPtr->nextPtr){
-	    Tcl_ListObjAppendElement(interp, resultPtr, elementPtr->objPtr);
+	if (unique) {
+	    for (; elementPtr != NULL; elementPtr = elementPtr->nextPtr){
+		if (elementPtr->count == 0) {
+		    Tcl_ListObjAppendElement(interp, resultPtr,
+			    elementPtr->objPtr);
+		}
+	    }
+	} else {
+	    for (; elementPtr != NULL; elementPtr = elementPtr->nextPtr){
+		Tcl_ListObjAppendElement(interp, resultPtr,
+			elementPtr->objPtr);
+	    }
 	}
     }
     ckfree((char*) elementArray);
@@ -2646,6 +2732,7 @@ MergeLists(leftPtr, rightPtr, infoPtr)
 {
     SortElement *headPtr;
     SortElement *tailPtr;
+    int cmp;
 
     if (leftPtr == NULL) {
         return rightPtr;
@@ -2653,20 +2740,28 @@ MergeLists(leftPtr, rightPtr, infoPtr)
     if (rightPtr == NULL) {
         return leftPtr;
     }
-    if (SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr) > 0) {
+    cmp = SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr);
+    if (cmp > 0) {
 	tailPtr = rightPtr;
 	rightPtr = rightPtr->nextPtr;
     } else {
+	if (cmp == 0) {
+	    leftPtr->count++;
+	}
 	tailPtr = leftPtr;
 	leftPtr = leftPtr->nextPtr;
     }
     headPtr = tailPtr;
     while ((leftPtr != NULL) && (rightPtr != NULL)) {
-	if (SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr) > 0) {
+	cmp = SortCompare(leftPtr->objPtr, rightPtr->objPtr, infoPtr);
+	if (cmp > 0) {
 	    tailPtr->nextPtr = rightPtr;
 	    tailPtr = rightPtr;
 	    rightPtr = rightPtr->nextPtr;
 	} else {
+	    if (cmp == 0) {
+		leftPtr->count++;
+	    }
 	    tailPtr->nextPtr = leftPtr;
 	    tailPtr = leftPtr;
 	    leftPtr = leftPtr->nextPtr;
@@ -2720,20 +2815,20 @@ SortCompare(objPtr1, objPtr2, infoPtr)
 
 	return order;
     }
-    if (infoPtr->index != -1) {
+    if (infoPtr->index != SORTIDX_NONE) {
 	/*
 	 * The "-index" option was specified.  Treat each object as a
 	 * list, extract the requested element from each list, and
-	 * compare the elements, not the lists.  The special index "end"
-	 * is signaled here with a large negative index.
+	 * compare the elements, not the lists.  "end"-relative indices
+	 * are signaled here with large negative values.
 	 */
 
 	if (Tcl_ListObjLength(infoPtr->interp, objPtr1, &listLen) != TCL_OK) {
 	    infoPtr->resultCode = TCL_ERROR;
 	    return order;
 	}
-	if (infoPtr->index < -1) {
-	    index = listLen - 1;
+	if (infoPtr->index < SORTIDX_NONE) {
+	    index = listLen + infoPtr->index + 1;
 	} else {
 	    index = infoPtr->index;
 	}
@@ -2759,8 +2854,8 @@ SortCompare(objPtr1, objPtr2, infoPtr)
 	    infoPtr->resultCode = TCL_ERROR;
 	    return order;
 	}
-	if (infoPtr->index < -1) {
-	    index = listLen - 1;
+	if (infoPtr->index < SORTIDX_NONE) {
+	    index = listLen + infoPtr->index + 1;
 	} else {
 	    index = infoPtr->index;
 	}
@@ -2782,10 +2877,10 @@ SortCompare(objPtr1, objPtr2, infoPtr)
 	order = DictionaryCompare(
 		Tcl_GetString(objPtr1),	Tcl_GetString(objPtr2));
     } else if (infoPtr->sortMode == SORTMODE_INTEGER) {
-	int a, b;
+	long a, b;
 
-	if ((Tcl_GetIntFromObj(infoPtr->interp, objPtr1, &a) != TCL_OK)
-		|| (Tcl_GetIntFromObj(infoPtr->interp, objPtr2, &b)
+	if ((Tcl_GetLongFromObj(infoPtr->interp, objPtr1, &a) != TCL_OK)
+		|| (Tcl_GetLongFromObj(infoPtr->interp, objPtr2, &b)
 		!= TCL_OK)) {
 	    infoPtr->resultCode = TCL_ERROR;
 	    return order;
@@ -2882,7 +2977,7 @@ static int
 DictionaryCompare(left, right)
     char *left, *right;          /* The strings to compare */
 {
-    Tcl_UniChar uniLeft, uniRight;
+    Tcl_UniChar uniLeft, uniRight, uniLeftLower, uniRightLower;
     int diff, zeros;
     int secondaryDiff = 0;
 
@@ -2954,32 +3049,30 @@ DictionaryCompare(left, right)
 	if ((*left != '\0') && (*right != '\0')) {
 	    left += Tcl_UtfToUniChar(left, &uniLeft);
 	    right += Tcl_UtfToUniChar(right, &uniRight);
+	    /*
+	     * Convert both chars to lower for the comparison, because
+	     * dictionary sorts are case insensitve.  Covert to lower, not
+	     * upper, so chars between Z and a will sort before A (where most
+	     * other interesting punctuations occur)
+	     */
+	    uniLeftLower = Tcl_UniCharToLower(uniLeft);
+	    uniRightLower = Tcl_UniCharToLower(uniRight);
 	} else {
 	    diff = UCHAR(*left) - UCHAR(*right);
 	    break;
 	}
 
-        diff = uniLeft - uniRight;
+        diff = uniLeftLower - uniRightLower;
         if (diff) {
+	    return diff;
+	} else if (secondaryDiff == 0) {
 	    if (Tcl_UniCharIsUpper(uniLeft) &&
 		    Tcl_UniCharIsLower(uniRight)) {
-		diff = Tcl_UniCharToLower(uniLeft) - uniRight;
-		if (diff) {
-		    return diff;
-                } else if (secondaryDiff == 0) {
-		    secondaryDiff = -1;
-                }
+		secondaryDiff = -1;
 	    } else if (Tcl_UniCharIsUpper(uniRight)
 		    && Tcl_UniCharIsLower(uniLeft)) {
-                diff = uniLeft - Tcl_UniCharToLower(uniRight);
-                if (diff) {
-		    return diff;
-                } else if (secondaryDiff == 0) {
-		    secondaryDiff = 1;
-                }
-            } else {
-                return diff;
-            }
+		secondaryDiff = 1;
+	    }
         }
     }
     if (diff == 0) {
