@@ -12,9 +12,10 @@
 # Copyright (c) 1998-1999 by Scriptics Corporation.
 # All rights reserved.
 # 
-# RCS: @(#) $Id: tcltest.tcl,v 1.22 2000/02/08 10:06:22 hobbs Exp $
+# RCS: @(#) $Id: tcltest.tcl,v 1.24.2.1 2001/08/09 01:00:57 dgp Exp $
 
-package provide tcltest 1.0
+package require Tcl 8.2
+package provide tcltest 1.0.1
 
 # create the "tcltest" namespace for all testing variables and procedures
 
@@ -170,8 +171,10 @@ namespace eval tcltest {
 
     if {![info exists mainThread]} {
 	variable mainThread 1
-	if {[info commands testthread] != {}} {
-	    set mainThread [testthread names]
+	if {[info commands thread::id] != {}} {
+	    set mainThread [thread::id]
+	} elseif {[info commands testthread] != {}} {
+	    set mainThread [testthread id]
 	}
     }
 
@@ -484,8 +487,6 @@ proc ::tcltest::initConstraints {} {
 	    "Windows 95"]
     set ::tcltest::testConstraints(98) [string equal $tcl_platform(os) \
 	    "Windows 98"]
-    set ::tcltest::testConstraints(win32s) [string equal $tcl_platform(os) \
-	    "Win32s"]
 
     # The following Constraints switches are used to mark tests that should
     # work, but have been temporarily disabled on certain platforms because
@@ -505,8 +506,6 @@ proc ::tcltest::initConstraints {} {
 
     set ::tcltest::testConstraints(pcCrash) \
 	    [expr {!$::tcltest::testConstraints(pc)}]
-    set ::tcltest::testConstraints(win32sCrash) \
-	    [expr {!$::tcltest::testConstraints(win32s)}]
     set ::tcltest::testConstraints(macCrash) \
 	    [expr {!$::tcltest::testConstraints(mac)}]
     set ::tcltest::testConstraints(unixCrash) \
@@ -1855,6 +1854,9 @@ proc ::tcltest::restore_locale {} {
 #	Returns the number of existing threads.
 proc ::tcltest::threadReap {} {
     if {[info commands testthread] != {}} {
+
+	# testthread built into tcltest
+
 	testthread errorproc ThreadNullError
 	while {[llength [testthread names]] > 1} {
 	    foreach tid [testthread names] {
@@ -1869,6 +1871,24 @@ proc ::tcltest::threadReap {} {
 	}
 	testthread errorproc ThreadError
 	return [llength [testthread names]]
+    } elseif {[info commands thread::id] != {}} {
+	
+	# Thread extension
+
+	thread::errorproc ThreadNullError
+	while {[llength [thread::names]] > 1} {
+	    foreach tid [thread::names] {
+		if {$tid != $::tcltest::mainThread} {
+		    catch {thread::send -async $tid {thread::exit}}
+		}
+	    }
+	    ## Enter a bit a sleep to give the threads enough breathing
+	    ## room to kill themselves off, otherwise the end up with a
+	    ## massive queue of repeated events
+	    after 1
+	}
+	thread::errorproc ThreadError
+	return [llength [thread::names]]
     } else {
 	return 1
     }
