@@ -12,7 +12,7 @@
  * See the file "License Terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclMacOSA.c,v 1.4 1999/05/11 07:12:36 jingham Exp $
+ * RCS: @(#) $Id: tclMacOSA.c,v 1.6.2.1 2001/04/04 21:22:19 hobbs Exp $
  */
 
 #define MAC_TCL
@@ -209,7 +209,7 @@ Tclapplescript_Init(
      * Perform the required stubs magic...
      */
      	
-    if (!Tcl_InitStubs(interp, "8.0", 0)) {
+    if (!Tcl_InitStubs(interp, "8.2", 0)) {
 	return TCL_ERROR;
     }
 
@@ -2120,7 +2120,7 @@ tclOSAStore(
 	    return TCL_ERROR;
 	}
 		
-	FSpCreateResFileCompat(&fileSpec,
+	FSpCreateResFileCompatTcl(&fileSpec,
 		'WiSH', 'osas', smSystemScript);	
 	myErr = ResError();
 	
@@ -2132,7 +2132,7 @@ tclOSAStore(
 	    goto rezEvalCleanUp;
 	}
 		
-	fileRef = FSpOpenResFileCompat(&fileSpec, fsRdWrPerm);
+	fileRef = FSpOpenResFileCompatTcl(&fileSpec, fsRdWrPerm);
 	if (fileRef == -1) {
 	    Tcl_AppendResult(interp, "Error reading the file: \"", 
 		    fileName, "\".", NULL);
@@ -2307,7 +2307,7 @@ tclOSALoad(
 	    return TCL_ERROR;
 	}
 			
-	fileRef = FSpOpenResFileCompat(&fileSpec, fsRdPerm);
+	fileRef = FSpOpenResFileCompatTcl(&fileSpec, fsRdPerm);
 	if (fileRef == -1) {
 	    Tcl_AppendResult(interp, "Error reading the file: \"", 
 		    fileName, "\".", NULL);
@@ -2695,9 +2695,10 @@ prepareScriptData(
     int i;
     char buffer[7];
     OSErr sysErr = noErr;
-		
+    Tcl_DString encodedText;
+
     Tcl_DStringInit(scrptData);
-	
+
     for (i = 0; i < argc; i++) {
 	Tcl_DStringAppend(scrptData, argv[i], -1);
 	Tcl_DStringAppend(scrptData, " ", 1);
@@ -2707,7 +2708,7 @@ prepareScriptData(
      * First replace the \n's with \r's in the script argument
      * Also replace "\\n" with "  ".
      */
-	 
+
     for (ptr = scrptData->string; *ptr != '\0'; ptr++) {
 	if (*ptr == '\n') {
 	    *ptr = '\r';
@@ -2718,10 +2719,13 @@ prepareScriptData(
 	    }
 	}
     }
- 	
-    sysErr = AECreateDesc(typeChar, Tcl_DStringValue(scrptData),
-	    Tcl_DStringLength(scrptData), scrptDesc);
-						
+
+    Tcl_UtfToExternalDString(NULL, Tcl_DStringValue(scrptData),
+	    Tcl_DStringLength(scrptData), &encodedText);
+    sysErr = AECreateDesc(typeChar, Tcl_DStringValue(&encodedText),
+	    Tcl_DStringLength(&encodedText), scrptDesc);
+    Tcl_DStringFree(&encodedText);
+
     if (sysErr != noErr) {
 	sprintf(buffer, "%6d", sysErr);
 	Tcl_DStringFree(scrptData);
@@ -2730,7 +2734,7 @@ prepareScriptData(
 	Tcl_DStringAppend(scrptData, " creating Script Data Descriptor.", 33);
 	return TCL_ERROR;					
     }
-	
+
     return TCL_OK;
 }
 
