@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclLoadShl.c,v 1.3 1999/04/16 00:48:04 stanton Exp $
+ * RCS: @(#) $Id: tclLoadShl.c,v 1.3.12.1 2001/09/12 21:26:36 dgp Exp $
  */
 
 #include <dl.h>
@@ -23,7 +23,7 @@
 #   undef EXTERN
 #endif
 
-#include "tcl.h"
+#include "tclInt.h"
 
 /*
  *----------------------------------------------------------------------
